using System.Text.Json;
using System.Text.RegularExpressions;
using HermesSocketLibrary.Requests.Messages;
using Serilog;

namespace TwitchChatTTS.Hermes.Socket.Requests
{
    public class UpdateTTSFilterAck : IRequestAck
    {
        public string Name => "update_tts_filter";
        private readonly User _user;
        private readonly JsonSerializerOptions _options;
        private readonly ILogger _logger;

        public UpdateTTSFilterAck(User user, JsonSerializerOptions options, ILogger logger)
        {
            _user = user;
            _options = options;
            _logger = logger;
        }

        public void Acknowledge(string requestId, string? json, IDictionary<string, object>? requestData)
        {
            var filter = JsonSerializer.Deserialize<TTSWordFilter>(json, _options);
            if (filter == null)
            {
                _logger.Warning($"TTS Filter data failed: null");
                return;
            }

            _logger.Debug($"Filter data [filter id: {filter.Id}][search: {filter.Search}][group id: {filter.Replace}]");
            var current = _user.RegexFilters.FirstOrDefault(f => f.Id == filter.Id);
            if (current == null)
            {
                _logger.Warning($"TTS Filter doest exist by id [filter id: {filter.Id}]");
                return;
            }

            current.Search = filter.Search;
            current.Replace = filter.Replace;
            current.Flag = filter.Flag;

            try
            {
                var re = new Regex(current.Search, ((RegexOptions)current.Flag) | RegexOptions.Compiled);
                re.Match(string.Empty);
                current.Regex = re;
            }
            catch (Exception) { }

            _logger.Information($"Filter has been updated [filter id: {filter.Id}]");
        }
    }
}